﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;                                                  ///< To use Toradex C Win32 DLLs


namespace vcsharp_i2c_demo
{

    unsafe public partial class I2CDemo : Form
    {
        public enum I2CSpeed { I2C_100KBPS, I2C_400KBPS, I2C_UNDEF_KBPS };

        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]                          ///< Importing DLL
        public static extern bool I2CInit();                                           ///< External function declaration
        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]
        public static extern void I2CDeInit();
        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]
        public static extern bool ReleaseI2CLock();
        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]
        public static extern bool GetI2CLock(Int32 timeout);
        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]
        public static extern bool I2CBurstRead(Byte uSlaveAddress, Byte[] pBuffer, short ucOffset, Int16 iNumberBytes);
        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]
        public static extern bool I2CBurstWrite(Byte uSlaveAddress, Byte[] pBuffer, short ucOffset, Int16 iNumberBytes);
        [DllImport("I2cLib_Dll.dll", CharSet = CharSet.Auto)]
        public static extern void I2CSetSpeed(I2CSpeed Speed);

        Byte i2cEepromAddress = 80;                                                     ///< EEPROM Device address 
        Byte i2cRTCAddress = 104;                                                       ///< RTC Device address                                  
        bool i2cError = true;
        uint bufferLenght = 0;                      

        public I2CDemo()
        {
            InitializeComponent();
            I2CInit();                                                                ///< Initialise I2C bus
            I2CSetSpeed(I2CSpeed.I2C_100KBPS);                                        ///< Set I2C bus speed
        }

        /// <summary>
        /// Event for Radio button to select automatic and manual read of RTC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AutoManualEvent(object sender, EventArgs e)
        {
            if (rdbtnManual.Checked == true)                                          ///< If manual selected disable automatic radio button and timer 
            {
                btnRTCRead.Enabled = true;
                rdbtnAuto.Checked = false;
                tmrRTCRead.Enabled = false;
            }
            else
            {
                btnRTCRead.Enabled = false;                                           ///< if manaul not selcted enable timer to read rtc on given interval and disable read button
                rdbtnManual.Checked = false;
                tmrRTCRead.Enabled = true;
            }
        }
        /// <summary>
        /// Read and Display Date, time and Day from RTC chip 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RtcReadOperation(object sender, EventArgs e)
        {
            short Seconds = 0;
            short Minutes = 0;
            short Hours = 0;
            short Day = 0;
            short Month = 0;
            int Year = 0;
            Byte[] tempData = new Byte[13];                             
            Byte[] i2cDataRead = new Byte[8];                                           ///< Read buffer
            GetI2CLock(5000);                                                           ///< Wait for 5000 ms to acquire I2C lock 
            I2CBurstRead(i2cRTCAddress, i2cDataRead, 0, 8);                             ///< Read date, time and day from RTC
            ReleaseI2CLock();                                                           ///< Release the I2C bus for other threads  

            ///<Obtained RTC data is in BCD format  
            tempData[0] = Convert.ToByte(i2cDataRead[0] & 0x0F);                        ///< Seconds 
            tempData[1] = Convert.ToByte(i2cDataRead[0] & 0x70);
            tempData[1] >>= 4;
                        
            tempData[2] = Convert.ToByte(i2cDataRead[1] & 0x0F);                        ///< Minutes
            tempData[3] = Convert.ToByte(i2cDataRead[1] & 0x70);
            tempData[3] >>= 4;

            tempData[4] = Convert.ToByte(i2cDataRead[2] & 0x0F);                        ///< Hours 
            tempData[5] = Convert.ToByte(i2cDataRead[2] & 0x30);
            tempData[5] >>= 4;

            tempData[6] = Convert.ToByte(i2cDataRead[3] & 0x07);                        ///< Day

            tempData[7] = Convert.ToByte(i2cDataRead[4] & 0x0F);                        ///< Date
            tempData[8] = Convert.ToByte(i2cDataRead[4] & 0x30);
            tempData[8] >>= 4;

            tempData[9] = Convert.ToByte(i2cDataRead[5] & 0x0F);                        ///< Month
            tempData[10] = Convert.ToByte(i2cDataRead[5] & 0x10);
            tempData[10] >>= 4;

            tempData[11] = Convert.ToByte(i2cDataRead[6] & 0x0F);                       ///< Year
            tempData[12] = Convert.ToByte(i2cDataRead[6] & 0xF0);
            tempData[12] >>= 4;

            Seconds = Convert.ToInt16(String.Concat(tempData[1], tempData[0]));
            Minutes = Convert.ToInt16(String.Concat(tempData[3], tempData[2]));
            Hours = Convert.ToInt16(String.Concat(tempData[5], tempData[4]));
            Day = Convert.ToInt16(String.Concat(tempData[8], tempData[7]));
            Month = Convert.ToInt16(String.Concat(tempData[10], tempData[9]));
            Year = Convert.ToInt16(String.Concat(tempData[12], tempData[11])) + 2000;

            DateTime dt = new DateTime(Year, Month, Day, Hours, Minutes, Seconds);
            txtRTCRead.Text = "Time (HH:MM:SS) : " + dt.ToLongTimeString() + Environment.NewLine +
                              "Date (DD/MM/YY)  : " + dt.ToShortDateString() + Environment.NewLine +
                              "Day is                   : " + dt.DayOfWeek.ToString();
        }
        /// <summary>
        /// Read from EEPROM starting adress 0x00
        /// Note: Write operation needs to be called before read operation for this demo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EepromRead(object sender, EventArgs e)
        {
            btnRTCRead.Enabled = true;                                                   ///< GUI code
            rdbtnAuto.Checked = false;
            tmrRTCRead.Enabled = false;
            rdbtnManual.Checked = true;
            lblMessage.Text = " ";

            Byte[] i2cDataAddress = new Byte[2] { 00, 00 };                              ///< First two bytes denote the EEPROM address where data will be written
            Byte[] i2cEepromRead = new Byte[bufferLenght];                               ///< Read buffer
            try
            {
                i2cError = GetI2CLock(5000);                                            ///< Wait for 5000 ms to acquire I2C lock 
                i2cError = I2CBurstWrite(i2cEepromAddress, i2cDataAddress, -1, 2);      ///< Writes the first two bytes of i2cDataWrite, i.e, the address to the EEPROM
                i2cError = I2CBurstRead(i2cEepromAddress, i2cEepromRead, -1, (short)bufferLenght);          ///< To read the bytes of data written at the given address (0x00)
                ReleaseI2CLock();                                                       ///< Release the I2C bus for other threads
                if (i2cError == false)
                    throw new Exception();                                              ///< If Read operation is not Successful
            }
            catch
            {
                lblMessage.Text = "Read failed:";
            }
            txtEepromRead.Text = Encoding.UTF8.GetString(i2cEepromRead, 0, (int)bufferLenght);
        }
        /// <summary>
        /// Write EEPROM starting from address 0x00
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void EepromWrite(object sender, EventArgs e)
        {
            btnRTCRead.Enabled = true;                                                      ///< GUI code
            rdbtnAuto.Checked = false;
            tmrRTCRead.Enabled = false;
            rdbtnManual.Checked = true;
            lblMessage.Text = " ";

            Byte[] i2cDataAddress = new Byte[2] { 00, 00 };                                 ///< First two bytes denote the EEPROM address where data will be written
            bufferLenght = (uint)txtEepromWrite.Text.Length;                                ///< Get lenght of text entered into textbox
            Byte[] i2cEepromWrite = new Byte[bufferLenght + 3];                             ///< Make buffer for writing to EEPROM Chip
            Byte[] temp = new Byte[bufferLenght];                                           ///< Temporary buffer to hold data from textbox
            temp = Encoding.ASCII.GetBytes(txtEepromWrite.Text);                            ///< Convert string to byte array
            Buffer.BlockCopy(i2cDataAddress, 0, i2cEepromWrite, 0, 2);                      ///< Copy EEPROM starting address 0x00 in starting of buffer
            Buffer.BlockCopy(temp, 0, i2cEepromWrite, 2, (int)bufferLenght);                ///< Concatenate data from textbox to buffer
            try
            {
                i2cError = GetI2CLock(5000);                                                ///< Wait for 5000 ms to acquire I2C lock 
                i2cError = I2CBurstWrite(i2cEepromAddress, i2cEepromWrite, -1, (short)(bufferLenght + 2));        ///< Write bytes on EEPROM 
                ReleaseI2CLock();                                                           ///< Release the I2C bus for other threads
                if (i2cError == false)
                    throw new Exception();
            }
            catch
            {
                lblMessage.Text = "Write failed";
            }
        }
        /// <summary>
        /// Form Close event 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void I2CDemo_Closed(object sender, EventArgs e)
        {
            I2CDeInit();                                                               ///< I2C deinit
        }
    }
}